@@ -9,6 +9,7 @@
 
 #include <QObject>
 #include <QVariantMap>
+#include <qqmlregistration.h>
 
 #include <memory>
 
@@ -36,6 +37,8 @@ class Intent;
 
 class QNetworkAccessManager;
 class QTemporaryDir;
+class QQmlEngine;
+class QJSEngine;
 
 class ApplicationController : public QObject
 {
@@ -56,6 +59,8 @@ public:
     explicit ApplicationController(QObject *parent = nullptr);
     ~ApplicationController() override;
 
+    static ApplicationController *create(QQmlEngine*, QJSEngine*);
+
     void setNetworkAccessManagerFactory(const std::function<QNetworkAccessManager *()> &namFactory);
 
     void requestOpenPage(const QString &page, const QVariantMap &properties = {});
@@ -157,4 +162,14 @@ private:
     bool m_importLock = false;
 };
 
+struct ApplicationControllerForeign
+{
+    Q_GADGET
+    QML_FOREIGN(ApplicationController)
+    QML_SINGLETON
+    QML_NAMED_ELEMENT(ApplicationController)
+public:
+    static ApplicationController *create(QQmlEngine*, QJSEngine*);
+};
+
 #endif // APPLICATIONCONTROLLER_H
