@@ -43,6 +43,7 @@
 
 #include <cassert>
 
+using namespace Qt::Literals;
 using namespace KItinerary;
 
 static constexpr const int POLL_COOLDOWN_ON_ERROR = 30; // seconds
@@ -73,21 +74,23 @@ LiveDataManager::LiveDataManager(QObject *parent)
             return;
         }
 
+        const auto journey = m_onboardStatus->journey();
+        if (journey.sections().empty()) {
+            return;
+        }
         for (const auto &resId : m_reservations) {
             auto res = m_resMgr->reservation(resId);
             if (!hasDeparted(resId, res) || hasArrived(resId, res)) {
                 continue;
             }
-            const auto journey = m_onboardStatus->journey();
-            if (journey.sections().empty()) {
-                return;
-            }
-            auto subjny = PublicTransportMatcher::subJourneyForReservation(res, journey.sections()[0]);
-            if (subjny.mode() == KPublicTransport::JourneySection::Invalid) {
-                return;
+
+            const auto lineData = ReservationHelper::lineNameAndNumber(res);
+            if (!PublicTransportMatcher::isSameMode(res, journey.sections()[0].route().line().mode()) ||
+                !PublicTransportMatcher::isSameRoute(journey.sections()[0].route(), lineData.first, lineData.second)) {
+                continue;
             }
 
-            updateJourneyData(subjny, resId, res);
+            applyJourney(resId, journey.sections()[0]);
         }
     });
 }
@@ -136,40 +139,51 @@ void LiveDataManager::setShowNotificationsOnLockScreen(bool enabled)
     m_showNotificationsOnLockScreen = enabled;
 }
 
+void LiveDataManager::setDownloadAssetsEnabled(bool enabled)
+{
+    m_downloadAssets = enabled;
+}
+
 KPublicTransport::Stopover LiveDataManager::arrival(const QString &resId) const
 {
-    return data(resId).arrival;
+    return data(resId).arrival();
 }
 
 KPublicTransport::Stopover LiveDataManager::departure(const QString &resId) const
 {
-    return data(resId).departure;
+    return data(resId).departure();
 }
 
 KPublicTransport::JourneySection LiveDataManager::journey(const QString &resId) const
 {
-    return data(resId).journey;
+    return data(resId).journey();
+}
+
+KPublicTransport::JourneySection LiveDataManager::trip(const QString &resId) const
+{
+    return data(resId).trip;
+}
+
+qsizetype LiveDataManager::tripDepartureIndex(const QString &resId) const
+{
+    return data(resId).departureIndex;
+}
+
+qsizetype LiveDataManager::tripArrivalIndex(const QString &resId) const
+{
+    return data(resId).arrivalIndex;
 }
 
 void LiveDataManager::setJourney(const QString &resId, const KPublicTransport::JourneySection &journey)
 {
     auto &ld = data(resId);
-    ld.journey = journey;
+    ld.trip = journey;
+    ld.departureIndex = 0;
+    ld.arrivalIndex = (qsizetype)journey.intermediateStops().size() + 1;
     ld.journeyTimestamp = now();
-    ld.departure = journey.departure();
-    ld.departureTimestamp = now();
-    ld.arrival = journey.arrival();
-    ld.arrivalTimestamp = now();
-    ld.store(resId, LiveData::AllTypes);
+    ld.store(resId);
 
     Q_EMIT journeyUpdated(resId);
-    Q_EMIT departureUpdated(resId);
-    Q_EMIT arrivalUpdated(resId);
-}
-
-void LiveDataManager::applyJourney(const QString &resId, const KPublicTransport::JourneySection &journey)
-{
-    updateJourneyData(journey, resId, m_resMgr->reservation(resId));
 }
 
 void LiveDataManager::checkForUpdates()
@@ -197,24 +211,27 @@ void LiveDataManager::checkReservation(const QVariant &res, const QString &resId
         return;
     }
 
-    auto jny = data(resId).journey;
+    auto jny = data(resId).journey();
     if (jny.mode() == JourneySection::Invalid) {
         jny.setMode(JourneySection::PublicTransport);
-        jny.setFrom(PublicTransport::locationFromPlace(LocationUtil::departureLocation(res), res));
-        jny.setTo(PublicTransport::locationFromPlace(LocationUtil::arrivalLocation(res), res));
+        jny.setFrom(PublicTransport::locationFromPlace(LocationUtil::departureLocation(res)));
+        jny.setTo(PublicTransport::locationFromPlace(LocationUtil::arrivalLocation(res)));
         jny.setScheduledDepartureTime(SortUtil::startDateTime(res));
         jny.setRoute(PublicTransport::routeForReservation(res));
     }
 
     TripRequest req(jny);
-    req.setDownloadAssets(true);
-    PublicTransport::selectBackends(req, m_ptMgr, res);
+    req.setDownloadAssets(m_downloadAssets);
+    if (!jny.from().hasCoordinate() || !jny.to().hasCoordinate()) {
+        // only consider ticket issuer codes when we don't have coordinates, those tend to be much more robust
+        PublicTransport::selectBackends(req, m_ptMgr, res);
+    }
     if (canSelectBackend(jny.from()) || canSelectBackend(jny.to()) || !req.backendIds().isEmpty() || jny.hasIdentifiers()) {
         auto reply = m_ptMgr->queryTrip(req);
         connect(reply, &Reply::finished, this, [this, resId, reply]() {
             reply->deleteLater();
             if (reply->error() == Reply::NoError) {
-                auto jny = reply->journeySection();
+                auto jny = reply->trip();
                 if (jny.mode() == JourneySection::PublicTransport) {
                     applyJourney(resId, jny);
                     return;
@@ -229,9 +246,7 @@ void LiveDataManager::checkReservation(const QVariant &res, const QString &resId
 
 void LiveDataManager::tripQueryFailed(const QString &resId)
 {
-    data(resId).setTimestamp(LiveData::Arrival, now());
-    data(resId).setTimestamp(LiveData::Departure, now());
-    data(resId).setTimestamp(LiveData::Journey, now());
+    data(resId).journeyTimestamp = now();
 }
 
 static KPublicTransport::Stopover applyLayoutData(const KPublicTransport::Stopover &stop, const KPublicTransport::Stopover &layout)
@@ -263,6 +278,14 @@ static void applyMissingStopoverData(KPublicTransport::Stopover &stop, const KPu
 
 static void applyMissingJourneyData(KPublicTransport::JourneySection &journey, const KPublicTransport::JourneySection &oldJny)
 {
+    // retain already existing vehicle/platform layout data if we are still departing/arriving in the same place
+    if (PublicTransport::isSameStopoverForLayout(oldJny.departure(), journey.departure())) {
+        journey.setDeparture(applyLayoutData(journey.departure(), oldJny.departure()));
+    }
+    if (PublicTransport::isSameStopoverForLayout(oldJny.arrival(), journey.arrival())) {
+        journey.setArrival(applyLayoutData(journey.arrival(), oldJny.arrival()));
+    }
+
     if (journey.intermediateStops().size() != oldJny.intermediateStops().size()) {
         return;
     }
@@ -273,6 +296,9 @@ static void applyMissingJourneyData(KPublicTransport::JourneySection &journey, c
             journey.setIntermediateStops(std::move(stops));
             return;
         }
+        if (PublicTransport::isSameStopoverForLayout(oldJny.intermediateStops()[i], stops[i])) {
+            stops[i] = applyLayoutData(stops[i], oldJny.intermediateStops()[i]);
+        }
         applyMissingStopoverData(stops[i], oldJny.intermediateStops()[i]);
     }
     journey.setIntermediateStops(std::move(stops));
@@ -296,38 +322,96 @@ static void applyMissingJourneyData(KPublicTransport::JourneySection &journey, c
     }
 }
 
-void LiveDataManager::updateJourneyData(const KPublicTransport::JourneySection &journey, const QString &resId, const QVariant &res)
+[[nodiscard]] static KPublicTransport::JourneySection expandJourney(const KPublicTransport::JourneySection &jny, const KPublicTransport::JourneySection &subJny, qsizetype beginIdx, qsizetype endIdx)
+{
+    KPublicTransport::JourneySection expJny = subJny;
+
+    auto stops = jny.intermediateStops();
+    if (beginIdx > 0) {
+        expJny.setDeparture(jny.departure());
+        stops[beginIdx - 1] = subJny.departure();
+    }
+    if (endIdx < (qsizetype)stops.size() + 1) {
+        expJny.setArrival(jny.arrival());
+        stops[endIdx - 1] = subJny.arrival();
+    }
+    stops.erase(stops.begin() + beginIdx, stops.begin() + endIdx - 1);
+    stops.insert(stops.begin() + beginIdx, subJny.intermediateStops().begin(), subJny.intermediateStops().end());
+    expJny.setIntermediateStops(std::move(stops));
+
+    return expJny;
+}
+
+void LiveDataManager::applyJourney(const QString &resId, const KPublicTransport::JourneySection &journey)
 {
     auto &ld = data(resId);
-    const auto oldDep = ld.departure;
-    const auto oldArr = ld.arrival;
-    const auto oldJny = ld.journey;
-    ld.journey = journey;
+    const auto oldDep = ld.departure();
+    const auto oldArr = ld.arrival();
+    auto oldJny = ld.trip;
 
-    // retain already existing vehicle/platform layout data if we are still departing/arriving in the same place
-    if (PublicTransport::isSameStopoverForLayout(ld.departure, journey.departure())) {
-        ld.journey.setDeparture(applyLayoutData(journey.departure(), ld.departure));
-    }
-    if (PublicTransport::isSameStopoverForLayout(ld.arrival, journey.arrival())) {
-        ld.journey.setArrival(applyLayoutData(journey.arrival(), ld.arrival));
-    }
-    applyMissingJourneyData(ld.journey, oldJny);
+    // align @p journey and @p ld.trip
+    // we can end up here in the following cases:
+    // (1) ld.trip is not set yet
+    // (2) ld.trip and journey are fully aligned
+    // (3) ld.trip is a subsection of journey
+    // (4) journey is a subsection of ld.trip
+    // we should get out of here with ld.trip either still being invalid, or with being aligned with journey
 
-    ld.journey.applyMetaData(true); // download logo assets if needed
+    if (!ld.isEmpty()) { // excludes case (1)
+        const auto oldInNewDep = journey.indexOfStopover(oldJny.departure());
+        const auto oldInNewArr = journey.indexOfStopover(oldJny.arrival());
+
+        if (oldInNewDep == 0 && oldInNewArr == (qsizetype)journey.intermediateStops().size() + 1) {
+            ld.trip = journey; // case (2)
+        } else if (oldInNewDep < 0 || oldInNewArr <= oldInNewDep) { // not case (3)
+            const auto newInOldDep = oldJny.indexOfStopover(journey.departure());
+            const auto newInOldArr = oldJny.indexOfStopover(journey.arrival());
+            if (newInOldDep == 0 && newInOldArr == (qsizetype)oldJny.intermediateStops().size() + 1) {
+                // case (2) again, symmetry violation in indexOfStopover
+                assert(false);
+            }
+            if (newInOldDep < 0 || newInOldArr <= newInOldDep) {
+                qCWarning(Log) << "Failed to align trip data, ignoring!";
+                return;
+            }
+            // case (4)
+            ld.trip = expandJourney(oldJny, journey, newInOldDep, newInOldArr);
+            ld.departureIndex = newInOldDep;
+            ld.arrivalIndex = newInOldArr;
+
+        } else { // case (3)
+            ld.trip = journey;
+            ld.departureIndex = oldInNewDep;
+            ld.arrivalIndex = oldInNewArr;
+            oldJny = expandJourney(journey, oldJny, oldInNewDep, oldInNewArr);
+        }
+    } else {
+        ld.trip = journey;
+    }
+
+    // retain still relevant information from the previous data
+    applyMissingJourneyData(ld.trip, oldJny);
+
+    // determine departure/arrival indexes
+    const auto res = m_resMgr->reservation(resId);
+    for (qsizetype i = 0; i <= (qsizetype)ld.trip.intermediateStops().size() + 1; ++i) {
+        if (PublicTransportMatcher::isDepartureForReservation(res, ld.trip.stopover(i))) {
+            ld.departureIndex = i;
+        }
+        if (PublicTransportMatcher::isArrivalForReservation(res, ld.trip.stopover(i))) {
+            ld.arrivalIndex = i;
+        }
+    }
+
+    ld.trip.applyMetaData(true); // download logo assets if needed
     ld.journeyTimestamp = now();
-    ld.departure = ld.journey.departure();
-    ld.departure.addNotes(oldDep.notes());
-    ld.departureTimestamp = now();
-    ld.arrival = ld.journey.arrival();
-    ld.arrival.setNotes(oldArr.notes());
-    ld.arrivalTimestamp = now();
-    ld.store(resId, LiveData::AllTypes);
+    ld.store(resId);
 
     // update reservation with live data
     std::vector<ReservationManager::ReservationChange> resUpdates;
     for (const auto &id : m_resMgr->reservationsForBatch(resId)) {
         const auto r = m_resMgr->reservation(id);
-        const auto newRes = PublicTransport::mergeJourney(r, ld.journey);
+        const auto newRes = PublicTransport::mergeJourney(r, ld.journey());
         if (!ReservationHelper::equals(r, newRes) && r.typeId() == newRes.typeId()) {
             resUpdates.emplace_back(id, newRes);
         }
@@ -337,12 +421,10 @@ void LiveDataManager::updateJourneyData(const KPublicTransport::JourneySection &
 
     // emit update signals
     Q_EMIT journeyUpdated(resId);
-    Q_EMIT departureUpdated(resId);
-    Q_EMIT arrivalUpdated(resId);
 
     // check if we need to notify
-    if (NotificationHelper::shouldNotify(oldDep, ld.journey.departure(), LiveData::Departure)
-        || NotificationHelper::shouldNotify(oldArr, ld.journey.arrival(), LiveData::Arrival)) {
+    if (NotificationHelper::shouldNotify(oldDep, ld.departure(), LiveData::Departure)
+        || NotificationHelper::shouldNotify(oldArr, ld.arrival(), LiveData::Arrival)) {
         showNotification(resId, ld);
     }
 }
@@ -390,7 +472,7 @@ void LiveDataManager::cancelNotification(const QString &resId)
 
 QDateTime LiveDataManager::departureTime(const QString &resId, const QVariant &res) const
 {
-    if (JsonLd::isA<TrainReservation>(res) || JsonLd::isA<BusReservation>(res)) {
+    if (JsonLd::isA<TrainReservation>(res) || JsonLd::isA<BusReservation>(res) || JsonLd::isA<BoatReservation>(res) || JsonLd::isA<FlightReservation>(res)) {
         const auto &dep = departure(resId);
         if (dep.hasExpectedDepartureTime()) {
             return dep.expectedDepartureTime();
@@ -402,7 +484,7 @@ QDateTime LiveDataManager::departureTime(const QString &resId, const QVariant &r
 
 QDateTime LiveDataManager::arrivalTime(const QString &resId, const QVariant &res) const
 {
-    if (JsonLd::isA<TrainReservation>(res) || JsonLd::isA<BusReservation>(res)) {
+    if (JsonLd::isA<TrainReservation>(res) || JsonLd::isA<BusReservation>(res) || JsonLd::isA<BoatReservation>(res) || JsonLd::isA<FlightReservation>(res)) {
         const auto &arr = arrival(resId);
         if (arr.hasExpectedArrivalTime()) {
             return arr.expectedArrivalTime();
@@ -443,8 +525,6 @@ void LiveDataManager::importData(const QString &resId, LiveData &&data)
     // we don't need to store data, Importer already does that
     m_data[resId] = std::move(data);
     Q_EMIT journeyUpdated(resId);
-    Q_EMIT departureUpdated(resId);
-    Q_EMIT arrivalUpdated(resId);
 }
 
 bool LiveDataManager::isRelevant(const QString &resId) const
@@ -463,6 +543,12 @@ bool LiveDataManager::isRelevant(const QString &resId) const
     if (JsonLd::isA<TrainReservation>(res) || JsonLd::isA<BusReservation>(res)) {
         return true;
     }
+    // handled by KPublicTransport but in a limited capacity, so only handle things that we originally got from there for now
+    if (JsonLd::isA<BoatReservation>(res) || JsonLd::isA<FlightReservation>(res)) {
+        if (journey(resId).mode() != KPublicTransport::JourneySection::Invalid) {
+            return true;
+        }
+    }
 
     // things with an updatable pkpass
     const auto passId = PkPassManager::passId(res);
@@ -498,23 +584,23 @@ void LiveDataManager::batchChanged(const QString &resId)
     const auto res = m_resMgr->reservation(resId);
     const auto dataIt = m_data.find(resId);
     if (dataIt != m_data.end()) {
-        if ((*dataIt).departureTimestamp.isValid() && !PublicTransportMatcher::isDepartureForReservation(res, (*dataIt).departure)) {
-            (*dataIt).departure = {};
-            (*dataIt).departureTimestamp = {};
-            (*dataIt).store(resId, LiveData::Departure);
-            Q_EMIT departureUpdated(resId);
-        }
-        if ((*dataIt).arrivalTimestamp.isValid() && !PublicTransportMatcher::isArrivalForReservation(res, (*dataIt).arrival)) {
-            (*dataIt).arrival = {};
-            (*dataIt).arrivalTimestamp = {};
-            (*dataIt).store(resId, LiveData::Arrival);
-            Q_EMIT arrivalUpdated(resId);
-        }
-
-        if ((*dataIt).journeyTimestamp.isValid() && !PublicTransportMatcher::isJourneyForReservation(res, (*dataIt).journey)) {
-            (*dataIt).journey = {};
-            (*dataIt).journeyTimestamp = {};
-            (*dataIt).store(resId, LiveData::Journey);
+        if ((*dataIt).journeyTimestamp.isValid() && !PublicTransportMatcher::isJourneyForReservation(res, (*dataIt).journey())) {
+            (*dataIt).departureIndex = -1;
+            (*dataIt).arrivalIndex = -1;
+            // check if the entire trip we have still applies, ie. just departure or arrival were changed
+            for (qsizetype i = 0; i <= (qsizetype)(*dataIt).trip.intermediateStops().size() + 1; ++i) {
+                if (PublicTransportMatcher::isDepartureForReservation(res, (*dataIt).trip.stopover(i))) {
+                    (*dataIt).departureIndex = i;
+                }
+                if (PublicTransportMatcher::isArrivalForReservation(res, (*dataIt).trip.stopover(i))) {
+                    (*dataIt).arrivalIndex = i;
+                }
+            }
+            if ((*dataIt).departureIndex < 0 || (*dataIt).arrivalIndex <= (*dataIt).departureIndex) {
+                (*dataIt).trip = {};
+                (*dataIt).journeyTimestamp = {};
+            }
+            (*dataIt).store(resId);
             Q_EMIT journeyUpdated(resId);
         }
     }
@@ -576,6 +662,11 @@ void LiveDataManager::pollBatchForUpdates(const QString &batchId, bool force)
     if (JsonLd::isA<TrainReservation>(res) || JsonLd::isA<BusReservation>(res)) {
         checkReservation(res, batchId);
     }
+    if (JsonLd::isA<BoatReservation>(res) || JsonLd::isA<FlightReservation>(res)) {
+        if (journey(batchId).mode() != KPublicTransport::JourneySection::Invalid) {
+            checkReservation(res, batchId);
+        }
+    }
 
     // check for pkpass updates, for each element in this batch
     const auto resIds = m_resMgr->reservationsForBatch(batchId);
@@ -650,23 +741,15 @@ int LiveDataManager::nextPollTimeForReservation(const QString &resId) const
 
     // check last poll time for this reservation
     const auto &ld = data(resId);
-    const auto lastArrivalPoll = ld.arrivalTimestamp;
-    const auto lastDeparturePoll = lastDeparturePollTime(resId, res);
-    auto lastRelevantPoll = lastArrivalPoll;
-    // ignore departure if we have already departed
-    if (!hasDeparted(resId, res) && lastDeparturePoll.isValid()) {
-        if (!lastArrivalPoll.isValid() || lastArrivalPoll > lastDeparturePoll) {
-            lastRelevantPoll = lastDeparturePoll;
-        }
-    }
+    const auto lastRelevantPoll = lastPollTime(resId, res);
     const int lastPollDist = (!lastRelevantPoll.isValid() || lastRelevantPoll > now) ? MAX_POLL_INTERVAL // no poll yet == long time ago
                                                                                      : lastRelevantPoll.secsTo(now);
     return std::max((it->pollInterval - lastPollDist) * 1000, pollCooldown(resId)); // we need msecs
 }
 
-QDateTime LiveDataManager::lastDeparturePollTime(const QString &batchId, const QVariant &res) const
+QDateTime LiveDataManager::lastPollTime(const QString &batchId, const QVariant &res) const
 {
-    auto dt = data(batchId).departureTimestamp;
+    auto dt = data(batchId).journeyTimestamp;
     if (dt.isValid()) {
         return dt;
     }
@@ -715,15 +798,8 @@ void LiveDataManager::pkPassUpdated(const QString &passId, const QStringList &ch
     }
 
     QString text = changes.join(QLatin1Char('\n'));
-
-    if (JsonLd::isA<FlightReservation>(passRes)) {
-        const auto flight = passRes.value<FlightReservation>().reservationFor().value<Flight>();
-
-        text.prepend(QLatin1Char('\n'));
-        text.prepend(i18n("Flight %1 to %2:",
-                          // TODO add formatter util for this.
-                          flight.airline().iataCode() + QLatin1Char(' ') + flight.flightNumber(),
-                          LocationUtil::name(LocationUtil::arrivalLocation(passRes))));
+    if (const auto title = ReservationHelper::label(passRes); !title.isEmpty()) {
+        text.prepend(title + '\n'_L1);
     }
 
     KNotification::event(KNotification::Notification, i18n("Itinerary change"), text, QLatin1StringView("clock"));
