@@ -14,11 +14,6 @@
 
 class ReservationManager;
 
-namespace SolidExtras
-{
-class NetworkStatus;
-}
-
 namespace KOSMIndoorMap
 {
 class MapLoader;
@@ -38,6 +33,10 @@ public:
 
     /** Trigger download of missing map data immediately, regardless of the current network state. */
     Q_INVOKABLE void download();
+    /** Triggers download of missing map data for the given reservations.
+     *  Executated immediately, regardless of the current network state.
+     */
+    Q_INVOKABLE void download(const QStringList &batchIds);
 
     void setAutomaticDownloadEnabled(bool enable);
 
@@ -48,9 +47,9 @@ Q_SIGNALS:
     void finished();
 
 private:
-    bool canAutoDownload() const;
+    [[nodiscard]] bool canAutoDownload() const;
     void addAutomaticRequestForBatch(const QString &batchId);
-    void addRequestForBatch(const QString &batchId);
+    void addRequestForBatch(const QString &batchId, bool force = false);
     void addRequest(double lat, double lon, const QDateTime &cacheUntil);
     void downloadNext();
     void downloadFinished();
@@ -69,7 +68,6 @@ private:
     Request m_currentRequest;
 
     KOSMIndoorMap::MapLoader *m_loader = nullptr;
-    SolidExtras::NetworkStatus *m_netStatus = nullptr;
 };
 
 #endif // MAPDOWNLOADMANAGER_H
