@@ -58,29 +58,19 @@ bool PublicTransport::isBoatMode(KPublicTransport::Line::Mode mode)
     }
 }
 
-KPublicTransport::Location PublicTransport::locationFromPlace(const QVariant &place, const QVariant &reservation)
+KPublicTransport::Location PublicTransport::locationFromPlace(const QVariant &place)
 {
     using namespace KItinerary;
 
     KPublicTransport::Location loc;
 
-    if (JsonLd::isA<FlightReservation>(reservation) || JsonLd::isA<TrainReservation>(reservation) || JsonLd::isA<BusReservation>(reservation)
-        || JsonLd::isA<BoatReservation>(reservation)) {
-        loc.setName(KItinerary::LocationUtil::name(place));
-    }
-    if (JsonLd::isA<TrainReservation>(reservation) || JsonLd::isA<BusReservation>(reservation)) {
+    loc.setName(KItinerary::LocationUtil::name(place));
+    if (JsonLd::isA<TrainStation>(place) || JsonLd::isA<BusStation>(place) || JsonLd::isA<BoatTerminal>(place)) {
         loc.setType(KPublicTransport::Location::Stop);
-    }
-    if (JsonLd::isA<LodgingReservation>(reservation)) {
-        loc.setName(reservation.value<LodgingReservation>().reservationFor().value<LodgingBusiness>().name());
+    } else if (!JsonLd::isA<Airport>(place)) {
         loc.setType(KPublicTransport::Location::Address);
     }
-    if (JsonLd::isA<EventReservation>(reservation)) {
-        loc.setName(reservation.value<EventReservation>().reservationFor().value<Event>().location().value<Place>().name());
-    }
-    if (JsonLd::isA<FoodEstablishmentReservation>(reservation)) {
-        loc.setName(reservation.value<FoodEstablishmentReservation>().reservationFor().value<FoodEstablishment>().name());
-    }
+
     const auto addr = KItinerary::LocationUtil::address(place);
     loc.setStreetAddress(addr.streetAddress());
     loc.setPostalCode(addr.postalCode());
@@ -265,22 +255,28 @@ QVariant PublicTransport::applyJourneySection(const QVariant &res, const KPublic
     return res;
 }
 
-QVariant PublicTransport::mergeDeparture(const QVariant &res, const KPublicTransport::Stopover &dep)
+template <typename LocT>
+static LocT mergeStopPoint(const KPublicTransport::Location &stopPoint, const LocT &stop)
+{
+    return KItinerary::MergeUtil::merge(PublicTransport::placeFromLocation<LocT>(stopPoint), stop).template value<LocT>();
+}
+
+static QVariant mergeDeparture(const QVariant &res, const KPublicTransport::Stopover &dep)
 {
     using namespace KItinerary;
 
     QVariant newRes;
     if (KPublicTransport::Line::modeIsRailBound(dep.route().line().mode())) {
         TrainTrip trip;
-        trip.setDepartureStation(placeFromLocation<TrainStation>(dep.stopPoint()));
+        trip.setDepartureStation(mergeStopPoint(dep.stopPoint(), res.value<TrainReservation>().reservationFor().value<TrainTrip>().departureStation()));
         trip.setDepartureTime(dep.scheduledDepartureTime());
         trip.setDeparturePlatform(dep.scheduledPlatform());
         TrainReservation trainRes;
         trainRes.setReservationFor(trip);
         newRes = trainRes;
-    } else if (isBusMode(dep.route().line().mode())) {
+    } else if (PublicTransport::isBusMode(dep.route().line().mode())) {
         BusTrip trip;
-        trip.setDepartureBusStop(placeFromLocation<BusStation>(dep.stopPoint()));
+        trip.setDepartureBusStop(mergeStopPoint(dep.stopPoint(), res.value<BusReservation>().reservationFor().value<BusTrip>().departureBusStop()));
         trip.setDepartureTime(dep.scheduledDepartureTime());
         trip.setDeparturePlatform(dep.scheduledPlatform());
         BusReservation busRes;
@@ -288,25 +284,25 @@ QVariant PublicTransport::mergeDeparture(const QVariant &res, const KPublicTrans
         newRes = busRes;
     }
 
-    return MergeUtil::merge(newRes, res);
+    return MergeUtil::merge(res, newRes);
 }
 
-QVariant PublicTransport::mergeArrival(const QVariant &res, const KPublicTransport::Stopover &arr)
+static QVariant mergeArrival(const QVariant &res, const KPublicTransport::Stopover &arr)
 {
     using namespace KItinerary;
 
     QVariant newRes;
     if (KPublicTransport::Line::modeIsRailBound(arr.route().line().mode())) {
         TrainTrip trip;
-        trip.setArrivalStation(placeFromLocation<TrainStation>(arr.stopPoint()));
+        trip.setArrivalStation(mergeStopPoint(arr.stopPoint(), res.value<TrainReservation>().reservationFor().value<TrainTrip>().arrivalStation()));
         trip.setArrivalTime(arr.scheduledArrivalTime());
         trip.setArrivalPlatform(arr.scheduledPlatform());
         TrainReservation trainRes;
         trainRes.setReservationFor(trip);
         newRes = trainRes;
-    } else if (isBusMode(arr.route().line().mode())) {
+    } else if (PublicTransport::isBusMode(arr.route().line().mode())) {
         BusTrip trip;
-        trip.setArrivalBusStop(placeFromLocation<BusStation>(arr.stopPoint()));
+        trip.setArrivalBusStop(mergeStopPoint(arr.stopPoint(), res.value<BusReservation>().reservationFor().value<BusTrip>().arrivalBusStop()));
         trip.setArrivalTime(arr.scheduledArrivalTime());
         trip.setArrivalPlatform(arr.scheduledPlatform());
         BusReservation busRes;
@@ -314,7 +310,7 @@ QVariant PublicTransport::mergeArrival(const QVariant &res, const KPublicTranspo
         newRes = busRes;
     }
 
-    return MergeUtil::merge(newRes, res);
+    return MergeUtil::merge(res, newRes);
 }
 
 QVariant PublicTransport::mergeJourney(const QVariant &res, const KPublicTransport::JourneySection &journey)
@@ -328,8 +324,7 @@ KPublicTransport::StopoverRequest PublicTransport::stopoverRequestForPlace(const
 {
     KPublicTransport::StopoverRequest req;
     req.setDateTime(std::max(dt, QDateTime::currentDateTime()));
-    req.setStop(PublicTransport::locationFromPlace(place, {}));
-    req.setDownloadAssets(true);
+    req.setStop(PublicTransport::locationFromPlace(place));
     return req;
 }
 
