@@ -16,6 +16,7 @@
 
 #include <QObject>
 #include <QVariant>
+#include <qqmlintegration.h>
 
 #include <chrono>
 
@@ -33,6 +34,8 @@ class DocumentManager;
 class TimelineDelegateController : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
+
     Q_PROPERTY(ReservationManager *reservationManager READ reservationManager WRITE setReservationManager NOTIFY setupChanged)
     Q_PROPERTY(LiveDataManager *liveDataManager READ liveDataManager WRITE setLiveDataManager NOTIFY setupChanged)
     Q_PROPERTY(TransferManager *transferManager READ transferManager WRITE setTransferManager NOTIFY setupChanged)
@@ -56,6 +59,8 @@ class TimelineDelegateController : public QObject
     Q_PROPERTY(qsizetype tripDepartureIndex READ tripDepartureIndex NOTIFY journeyChanged)
     Q_PROPERTY(qsizetype tripArrivalIndex READ tripArrivalIndex NOTIFY journeyChanged)
 
+    /** Planned start time of this element. */
+    Q_PROPERTY(QDateTime startTime READ startTime NOTIFY journeyChanged)
     /** Effective end time, ie. our best knowledge of arriving at the destination (for transit elements), or
      *  ending of the associated event for non-transit elements.
      */
@@ -122,6 +127,7 @@ public:
     [[nodiscard]] qsizetype tripDepartureIndex() const;
     [[nodiscard]] qsizetype tripArrivalIndex() const;
 
+    [[nodiscard]] QDateTime startTime() const;
     [[nodiscard]] QDateTime effectiveEndTime() const;
 
     [[nodiscard]] bool isLocationChange() const;
