@@ -7,6 +7,8 @@
 #ifndef TIMELINEELEMENT_H
 #define TIMELINEELEMENT_H
 
+#include "qmlregistrationhelper.h"
+
 #include <QDateTime>
 #include <QString>
 #include <QVariant>
@@ -23,6 +25,7 @@ class GeoCoordinates;
 class TimelineElement
 {
     Q_GADGET
+    QML_VALUE_TYPE(timelineElement)
 
 public:
     // Note: the order in here defines the priority of element if they occur at the same time
@@ -114,5 +117,6 @@ private:
 };
 
 Q_DECLARE_METATYPE(TimelineElement)
+REGISTER_GADGET_ENUMS(TimelineElement)
 
 #endif // TIMELINEELEMENT_H
