@@ -9,7 +9,6 @@
 #include "locationinformation.h"
 #include "reservationmanager.h"
 #include "transfermanager.h"
-#include "tripgroup.h"
 #include "tripgroupmanager.h"
 #include "weatherinformation.h"
 
@@ -102,7 +101,6 @@ void TimelineModel::setReservationManager(ReservationManager *mgr)
     connect(mgr, &ReservationManager::batchAdded, this, &TimelineModel::batchAdded);
     connect(mgr, &ReservationManager::batchChanged, this, &TimelineModel::batchChanged);
     connect(mgr, &ReservationManager::batchContentChanged, this, &TimelineModel::batchChanged);
-    connect(mgr, &ReservationManager::batchRenamed, this, &TimelineModel::batchRenamed);
     connect(mgr, &ReservationManager::batchRemoved, this, &TimelineModel::batchRemoved);
     Q_EMIT setupChanged();
     QMetaObject::invokeMethod(this, &TimelineModel::populate, Qt::QueuedConnection);
@@ -132,7 +130,6 @@ void TimelineModel::setTripGroupManager(TripGroupManager *mgr)
     }
 
     m_tripGroupManager = mgr;
-    connect(mgr, &TripGroupManager::tripGroupAdded, this, &TimelineModel::tripGroupAdded);
     connect(mgr, &TripGroupManager::tripGroupChanged, this, &TimelineModel::tripGroupChanged);
     connect(mgr, &TripGroupManager::tripGroupRemoved, this, &TimelineModel::tripGroupRemoved);
     Q_EMIT setupChanged();
@@ -239,16 +236,6 @@ QVariant TimelineModel::data(const QModelIndex &index, int role) const
         std::sort(v.begin(), v.end(), SortUtil::isBefore);
         return QVariant::fromValue(v);
     }
-    case TripGroupIdRole:
-        if (elem.elementType == TimelineElement::TripGroup) {
-            return elem.content();
-        }
-        break;
-    case TripGroupRole:
-        if (elem.elementType == TimelineElement::TripGroup) {
-            return QVariant::fromValue(m_tripGroupManager->tripGroup(elem.content().toString()));
-        }
-        break;
     case TransferRole:
         if (elem.elementType == TimelineElement::Transfer) {
             return elem.content();
@@ -278,8 +265,6 @@ QHash<int, QByteArray> TimelineModel::roleNames() const
     names.insert(LocationInformationRole, "locationInformation");
     names.insert(WeatherForecastRole, "weatherInformation");
     names.insert(ReservationsRole, "reservations");
-    names.insert(TripGroupIdRole, "tripGroupId");
-    names.insert(TripGroupRole, "tripGroup");
     names.insert(TransferRole, "transfer");
     return names;
 }
@@ -294,39 +279,23 @@ int TimelineModel::todayRow() const
 
 void TimelineModel::populate()
 {
-    if (m_isPopulated || !m_resMgr || !m_transferManager || !m_tripGroupManager) {
+    if (m_isPopulated || !m_resMgr || !m_transferManager || !m_tripGroupManager || m_tripGroupId.isEmpty()) {
         return;
     }
     m_isPopulated = true;
 
     beginResetModel();
-    if (m_tripGroupId.isEmpty()) {
-        for (const auto &resId : m_resMgr->batches()) {
-            populateReservation(resId);
-        }
-    } else {
-        m_tripGroup = m_tripGroupManager->tripGroup(m_tripGroupId);
-        const auto elems = m_tripGroup.elements();
-        for (const auto &resId : elems) {
-            populateReservation(resId);
-        }
+    m_tripGroup = m_tripGroupManager->tripGroup(m_tripGroupId);
+    const auto elems = m_tripGroup.elements();
+    for (const auto &resId : elems) {
+        populateReservation(resId);
     }
     std::sort(m_elements.begin(), m_elements.end());
     endResetModel();
 
     // load existing transfers and trip groups into the model
-    if (m_tripGroupId.isEmpty()) {
-        for (const auto &batchId : m_resMgr->batches()) {
-            updateTransfersForBatch(batchId);
-        }
-        for (const auto &group : m_tripGroupManager->tripGroups()) {
-            tripGroupAdded(group);
-        }
-    } else {
-        const auto elems = m_tripGroup.elements();
-        for (const auto &batchId : elems) {
-            updateTransfersForBatch(batchId);
-        }
+    for (const auto &batchId : elems) {
+        updateTransfersForBatch(batchId);
     }
 
     updateTodayMarker();
@@ -351,7 +320,7 @@ void TimelineModel::populateReservation(const QString &resId)
 
 void TimelineModel::batchAdded(const QString &resId)
 {
-    if (!m_isPopulated || (!m_tripGroupId.isEmpty() && !m_tripGroup.elements().contains(resId))) {
+    if (!m_isPopulated || !m_tripGroup.elements().contains(resId)) {
         return;
     }
 
@@ -406,7 +375,7 @@ std::vector<TimelineElement>::iterator TimelineModel::insertOrUpdate(std::vector
 
 void TimelineModel::batchChanged(const QString &resId)
 {
-    if (!m_isPopulated || (!m_tripGroupId.isEmpty() && !m_tripGroup.elements().contains(resId))) {
+    if (!m_isPopulated || !m_tripGroup.elements().contains(resId)) {
         return;
     }
 
@@ -421,27 +390,6 @@ void TimelineModel::batchChanged(const QString &resId)
     updateInformationElements();
 }
 
-void TimelineModel::batchRenamed(const QString &oldBatchId, const QString &newBatchId)
-{
-    if (!m_isPopulated || !m_tripGroupId.isEmpty()) {
-        return;
-    }
-
-    for (auto it = m_elements.begin(); it != m_elements.end(); ++it) {
-        if (!(*it).isReservation() || (*it).batchId() != oldBatchId) {
-            continue;
-        }
-
-        (*it).setContent(newBatchId);
-        const auto idx = index((int)std::distance(m_elements.begin(), it), 0);
-        Q_EMIT dataChanged(idx, idx);
-
-        if ((*it).rangeType == TimelineElement::SelfContained || (*it).rangeType == TimelineElement::RangeEnd) {
-            break;
-        }
-    }
-}
-
 void TimelineModel::updateElement(const QString &resId, const QVariant &res, TimelineElement::RangeType rangeType)
 {
     const auto it = std::find_if(m_elements.begin(), m_elements.end(), [resId, rangeType](const auto &e) {
@@ -525,7 +473,7 @@ void TimelineModel::updateTodayMarker()
     }
 
     const auto oldRow = todayRow();
-    if (oldRow >= newRow) {
+    if (oldRow >= newRow && newRow >= 0) {
         return;
     }
 
@@ -572,6 +520,10 @@ void TimelineModel::updateInformationElements()
 
         auto newCountry = homeCountry;
         newCountry.setIsoCode(LocationUtil::address((*it).destination()).addressCountry());
+        if (!previousCountry.timeZone().isValid() && (*it).isLocationChange()) {
+            // if we don't know the previous timezone, start with the departure location of a location change
+            previousCountry.setTimeZone(timeZone((*it).dt), (*it).dt);
+        }
         newCountry.setTimeZone(previousCountry.timeZone(), (*it).dt);
         newCountry.setTimeZone(timeZone((*it).endDateTime()), (*it).dt);
         if (newCountry == previousCountry) {
@@ -662,14 +614,18 @@ void TimelineModel::updateWeatherElements()
 
     auto date = now();
     if (m_tripGroup.beginDateTime().isValid()) {
-        date = std::max(date, m_tripGroup.beginDateTime());
+        auto tgStartOfDay = m_tripGroup.beginDateTime();
+        tgStartOfDay.setTime({0, 0});
+        date = std::max(date, tgStartOfDay);
     }
     // round to next full hour
     date.setTime(QTime(date.time().hour(), 0));
-    date = date.addSecs(60 * 60);
-    auto maxForecastTime = m_weatherMgr->maximumForecastTime(date.date());
+    date = date.addDuration(std::chrono::hours(1));
+    auto maxForecastTime = m_weatherMgr->maximumForecastTime(today());
     if (m_tripGroup.endDateTime().isValid()) {
-        maxForecastTime = std::min(maxForecastTime, m_tripGroup.endDateTime());
+        auto tgEndOfDay = m_tripGroup.endDateTime();
+        tgEndOfDay.setTime({23, 59, 59});
+        maxForecastTime = std::min(maxForecastTime, tgEndOfDay);
     }
 
     // look through the past, clean up weather elements there and figure out where we are
@@ -836,80 +792,38 @@ void TimelineModel::setCurrentDateTime(const QDateTime &dt)
     }
 }
 
-void TimelineModel::tripGroupAdded(const QString &groupId)
-{
-    if (!m_isPopulated || !m_tripGroupId.isEmpty()) {
-        return;
-    }
-
-    const auto g = m_tripGroupManager->tripGroup(groupId);
-    if (g.elements().size() < 2) {
-        return;
-    }
-
-    TimelineElement beginElem{this, TimelineElement::TripGroup, g.beginDateTime(), groupId};
-    beginElem.rangeType = TimelineElement::RangeBegin;
-    const auto startRow = insertElement(std::move(beginElem));
-
-    TimelineElement endElem{this, TimelineElement::TripGroup, g.endDateTime(), groupId};
-    endElem.rangeType = TimelineElement::RangeEnd;
-    const auto endRow = insertElement(std::move(endElem));
-
-    // elements in that group now have a trip group id
-    if (endRow - startRow > 1) {
-        const auto beginIdx = index(startRow + 1, 0);
-        const auto endIdx = index(endRow - 1, 0);
-        Q_EMIT dataChanged(beginIdx, endIdx, {TimelineModel::TripGroupIdRole});
-    }
-}
-
 void TimelineModel::tripGroupChanged(const QString &groupId)
 {
-    if (!m_isPopulated) {
+    if (!m_isPopulated || m_tripGroupId != groupId) {
         return;
     }
 
-    if (m_tripGroupId.isEmpty()) {
-        // ### this can be done better probably
-        tripGroupRemoved(groupId);
-        tripGroupAdded(groupId);
-    } else if (m_tripGroupId == groupId) {
-        const auto oldElems = m_tripGroup.elements();
-        m_tripGroup = m_tripGroupManager->tripGroup(groupId);
-        const auto newElems = m_tripGroup.elements();
+    const auto oldElems = m_tripGroup.elements();
+    const auto tg = m_tripGroupManager->tripGroup(groupId);
+    const auto timesChanged = tg.beginDateTime() != m_tripGroup.beginDateTime() || tg.endDateTime() != m_tripGroup.endDateTime();
+    m_tripGroup = tg;
+    const auto newElems = m_tripGroup.elements();
 
-        for (const auto &oldBatchId : oldElems) {
-            if (!newElems.contains(oldBatchId)) {
-                batchRemoved(oldBatchId);
-            }
+    for (const auto &oldBatchId : oldElems) {
+        if (!newElems.contains(oldBatchId)) {
+            batchRemoved(oldBatchId);
         }
-        for (const auto &newBatchId : newElems) {
-            if (!oldElems.contains(newBatchId)) {
-                batchAdded(newBatchId);
-            }
+    }
+    for (const auto &newBatchId : newElems) {
+        if (!oldElems.contains(newBatchId)) {
+            batchAdded(newBatchId);
         }
     }
+
+    // needed when start or end time changed
+    if (timesChanged) {
+        updateTodayMarker();
+    }
 }
 
 void TimelineModel::tripGroupRemoved(const QString &groupId)
 {
-    if (!m_isPopulated) {
-        return;
-    }
-
-    if (m_tripGroupId.isEmpty()) {
-        for (auto it = m_elements.begin(); it != m_elements.end();) {
-            if ((*it).elementType != TimelineElement::TripGroup || (*it).content().toString() != groupId) {
-                ++it;
-                continue;
-            }
-
-            const auto row = (int)std::distance(m_elements.begin(), it);
-            beginRemoveRows({}, row, row);
-            it = m_elements.erase(it);
-            endRemoveRows();
-        }
-    } else if (m_tripGroupId == groupId) {
+    if (m_isPopulated && m_tripGroupId == groupId) {
         beginResetModel();
         m_elements.clear();
         m_isPopulated = false;
