@@ -7,6 +7,8 @@
 #ifndef TRANSFER_H
 #define TRANSFER_H
 
+#include "qmlregistrationhelper.h"
+
 #include <KPublicTransport/Journey>
 #include <KPublicTransport/Location>
 
@@ -25,6 +27,8 @@ class QJsonObject;
 class Transfer
 {
     Q_GADGET
+    QML_VALUE_TYPE(transfer)
+
     Q_PROPERTY(Alignment alignment READ alignment WRITE setAlignment)
     Q_PROPERTY(State state READ state WRITE setState)
     Q_PROPERTY(FloatingLocationType floatingLocationType READ floatingLocationType WRITE setFloatingLocationType)
@@ -102,6 +106,11 @@ public:
     /** Unique identifier usable naming the file to store this into. */
     [[nodiscard]] QString identifier() const;
     [[nodiscard]] static QString identifier(const QString &resId, Transfer::Alignment alignment);
+    struct Identifier {
+        QString id;
+        Transfer::Alignment alignment;
+    };
+    [[nodiscard]] static Identifier parseIdentifier(QStringView id);
 
     /** Check if transfers are considered identical
      *  in the sense that no update is required.
@@ -119,4 +128,6 @@ private:
 
 Q_DECLARE_METATYPE(Transfer)
 
+REGISTER_GADGET_ENUMS(Transfer)
+
 #endif // TRANSFER_H
