@@ -14,16 +14,17 @@
 
 #include <QUrl>
 
-namespace KAndroidExtras {
+namespace KAndroidExtras
+{
 
 /** Conversion methods for android.net.Uri. */
 namespace Uri
 {
-    /** Create an android.net.Uri from a QUrl. */
-    KANDROIDEXTRAS_EXPORT QJniObject fromUrl(const QUrl &url);
+/** Create an android.net.Uri from a QUrl. */
+KANDROIDEXTRAS_EXPORT QJniObject fromUrl(const QUrl &url);
 
-    /** Convert a android.net.Uri to a QUrl. */
-    KANDROIDEXTRAS_EXPORT QUrl toUrl(const QJniObject &uri);
+/** Convert a android.net.Uri to a QUrl. */
+KANDROIDEXTRAS_EXPORT QUrl toUrl(const QJniObject &uri);
 }
 
 JNI_DECLARE_CONVERTER(android::net::Uri, QUrl, (Uri::toUrl(value)), (Uri::fromUrl(value)))
